﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Text.RegularExpressions;

namespace WowToggler.App
{
    public class WindowHandler
    {
        private Dictionary<int, int> _handles;
        public Dictionary<int, int> FindHandles()
        {
            _handles = new Dictionary<int, int>();

            var windowFinder = new WindowFinder();
            windowFinder.FindWindows(0, null, null, "wow-64", FoundWindow);
            windowFinder.FindWindows(0, null, null, "wow", FoundWindow);

            return _handles.OrderBy(x =>
                                        {
                                            try
                                            {
                                                return Process.GetProcessById(x.Key).StartTime;
                                            }
                                            catch
                                            {
                                                return DateTime.MinValue;
                                            }
                                        }).ToDictionary(y => y.Key, y => y.Value);
        }

        private bool FoundWindow(int handle, int processId)
        {
            if(!_handles.ContainsKey(processId))
                _handles.Add(processId, handle);

            return true;
        }
    }

    public class WindowFinder
    {
        // Win32 constants.
        const int WM_GETTEXT = 0x000D;
        const int WM_GETTEXTLENGTH = 0x000E;

        [DllImport("User32.Dll")]
        private static extern void GetClassName(int hWnd, StringBuilder s, int nMaxCount);

        [DllImport("User32.dll")]
        private static extern int GetWindowText(int hWnd, StringBuilder text, int count);

        [DllImport("User32.dll")]
        private static extern Int32 SendMessage(int hWnd, int Msg, int wParam, StringBuilder lParam);

        [DllImport("User32.dll")]
        private static extern Int32 SendMessage(int hWnd, int Msg, int wParam, int lParam);

        [DllImport("user32")]
        private static extern int GetWindowThreadProcessId(int hWnd, out int lpdwProcessId);

        [DllImport("user32.Dll")]
        private static extern Boolean EnumChildWindows(int hWndParent, PChildCallBack lpEnumFunc, int lParam);

        private delegate bool PChildCallBack(int hWnd, int lParam);

        private event FoundWindowCallback foundWindow;
        public delegate bool FoundWindowCallback(int hWnd, int processId);

        private int parentHandle;
        private Regex className;
        private Regex windowText;
        private string process;

        public void FindWindows(int parentHandle, Regex className, Regex windowText, string process, FoundWindowCallback fwc)
        {
            this.parentHandle = parentHandle;
            this.className = className;
            this.windowText = windowText;
            this.process = process;

            foundWindow = fwc;

            EnumChildWindows(parentHandle, new PChildCallBack(enumChildWindowsCallback), 0);
        }

        private bool enumChildWindowsCallback(int handle, int lParam)
        {
            if (className != null)
            {
                StringBuilder sbClass = new StringBuilder(256);
                GetClassName(handle, sbClass, sbClass.Capacity);

                if (!className.IsMatch(sbClass.ToString()))
                    return true;
            }

            if (windowText != null)
            {
                int txtLength = SendMessage(handle, WM_GETTEXTLENGTH, 0, 0);
                StringBuilder sbText = new StringBuilder(txtLength + 1);
                SendMessage(handle, WM_GETTEXT, sbText.Capacity, sbText);

                if (!windowText.IsMatch(sbText.ToString()))
                    return true;
            }

            if (process != null)
            {
                int processID;
                GetWindowThreadProcessId(handle, out processID);

                Process p = Process.GetProcessById(processID);

                if (process.ToLower() != p.ProcessName.ToLower())
                    return true;
            }

            int processIDForRef;
                GetWindowThreadProcessId(handle, out processIDForRef);

            return foundWindow(handle, processIDForRef);
        }
    }
}
