﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using WowToggler.App;

namespace WowToggler
{
    public partial class Form1 : Form
    {
        const int SW_HIDE = 0;
        const int SW_SHOW = 5;
        private Timer _checkForNewHandlesTimer;
        private readonly WindowHandler _windowHandler;
        private List<int> _hiddenHandles = new List<int>();

        public Form1()
        {
            _windowHandler = new WindowHandler();
            InitializeComponent();
            SearchAndAddWindows();
            InitTimer();
        }

        public void InitTimer()
        {
            _checkForNewHandlesTimer = new Timer();
            _checkForNewHandlesTimer.Tick += CheckForNewHandlesTimerTick;
            _checkForNewHandlesTimer.Interval = 10000;
            _checkForNewHandlesTimer.Start();
        }

        private void CheckForNewHandlesTimerTick(object sender, EventArgs e)
        {
            SearchAndAddWindows();
        }

        private string GetBtnText(int handleId)
        {
            if (_hiddenHandles.Contains(handleId))
            {
                return "Show ID: " + handleId;
            }
            else
            {
                return "Hide ID: " + handleId;
            }
        }


        private void SearchAndAddWindows()
        {
            var handles = _windowHandler.FindHandles();
            flPanel.Controls.Clear();

            foreach (var handle in handles)
            {
                var btn = new Button
                              {
                                  Name = handle.Value.ToString(),
                                  Text = GetBtnText(handle.Value),
                                  Size = new Size(150, 50)
                              };
                btn.Click += BtnClick;
                flPanel.Controls.Add(btn);
            }

            if(flPanel.Controls.Count == 0)
            {
                flPanel.Controls.Add(new Label
                                         {
                                             Text = "No WoW windows found!",
                                             Size = new Size(200, 50)
                                         });
            }
        }
        
        private void BtnClick(object sender, EventArgs e)
        {
            ToggleWindow(((Button)sender));
        }
        
        private void ToggleWindow(Button btn)
        {
            var handleId = int.Parse(btn.Name);

            var wasVisible = ShowWindow(handleId, SW_HIDE);

            if (!wasVisible)
            {
                ShowWindow(handleId, SW_SHOW);

                if (_hiddenHandles.Contains(handleId))
                    _hiddenHandles.Remove(handleId);
            }
            else
            {
                _hiddenHandles.Add(handleId);
            }

            btn.Text = GetBtnText(handleId);
        }

        public void Form1_Resize(object sender, EventArgs e)
        {
            if (FormWindowState.Minimized == this.WindowState)
            {
                notifyIcon1.Visible = true;                
                this.Hide();
            }
            else if (FormWindowState.Normal == this.WindowState)
            {
                notifyIcon1.Visible = false;
            }
        }

        [DllImport("user32.dll")]
        public static extern IntPtr FindWindow(String sClassName, String sAppName);

        [DllImport("user32.dll")]
        public static extern Boolean ShowWindow(int hWnd, Int32 nCmdShow);

        private void notifyIcon1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            this.Show();
            this.Focus();
        }
    }
}
